@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -38,7 +38,7 @@ package nu.xom;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b5
+ * @version 1.2b2
  * 
  */
 public class Attribute extends Node {
@@ -149,13 +149,10 @@ public class Attribute extends Node {
 
         prefix = "";
         String localName = name;
-        // ???? Warning: do not cache the value returned by indexOf here
-        // and in build
-        // without profiling. My current tests show doing this slows 
-        // down the parse by about 7%. I can't explain it.
-        if (name.indexOf(':') > 0) {
-            prefix = name.substring(0, name.indexOf(':'));   
-            localName = name.substring(name.indexOf(':') + 1);
+        int prefixPosition = name.indexOf(':');
+        if (prefixPosition > 0) {
+            prefix = name.substring(0, prefixPosition);   
+            localName = name.substring(prefixPosition + 1);
         }
 
         try {
@@ -167,7 +164,7 @@ public class Attribute extends Node {
         }
         _setNamespace(prefix, URI);
         _setValue(value);
-        if ("xml".equals(this.prefix) && "id".equals(this.localName)) {
+        if (isXMLID()) {
             _setType(Attribute.Type.ID);
         }   
         else {
@@ -203,17 +200,15 @@ public class Attribute extends Node {
       String qualifiedName, String URI, String value, Type type, String localName) {
         
         Attribute result = new Attribute();
-    
         String prefix = "";
-        if (qualifiedName.indexOf(':') >= 0) {
-            prefix = qualifiedName.substring(0, qualifiedName.indexOf(':'));
+        int prefixPosition = qualifiedName.indexOf(':');     
+        if (prefixPosition >= 0) {
+            prefix = qualifiedName.substring(0, prefixPosition);
             if ("xml:id".equals(qualifiedName)) {
                 type = Attribute.Type.ID;
                 value = normalize(value);
-                // ???? should I only do this if validating?
-                // Verifier.checkNCName(value);
             }
-        }        
+        }   
         
         result.localName = localName;
         result.prefix = prefix;
@@ -277,9 +272,15 @@ public class Attribute extends Node {
      * </p>
      * 
      * @param type the DTD type of this attribute
+     * @throws NullPointerException if <code>type</code> is null
+     * @throws IllegalDataException if this is an <code>xml:id</code>
+     *     attribute and the <code>type</code> is not ID
      */
     public void setType(Type type) {
         
+        if (type == null) {
+            throw new NullPointerException("Null attribute type");
+        }
         if (isXMLID() && ! Type.ID.equals(type)) {
             throw new IllegalDataException(
               "Can't change type of xml:id attribute to " + type);
@@ -323,13 +324,11 @@ public class Attribute extends Node {
      * </p>
      * 
      * @param value the value assigned to the attribute
+     * 
      * @throws IllegalDataException if the value contains characters 
      *     which are not legal in XML such as vertical tab or a null. 
      *     Characters such as " and &amp; are legal, but will be 
      *     automatically escaped when the attribute is serialized.
-     * @throws MalformedURIException if this is an 
-     *     <code>xml:base</code> attribute, and the value is not a
-     *     legal IRI
      */
     public void setValue(String value) {
         _setValue(value);
@@ -337,17 +336,11 @@ public class Attribute extends Node {
 
     
     private void _setValue(String value) {
-        
-        if ("xml".equals(this.prefix) && "id".equals(this.localName)) {
-            // ???? do I really want to do this. XML ID test case
-            // suggests not
-            Verifier.checkNCName(value);
-        }
-        else {
-            Verifier.checkPCDATA(value);
+        Verifier.checkPCDATA(value);
+        if (this.isXMLID()) {
+            value = normalize(value);
         }
         this.value = value;
-        
     }
 
 
@@ -468,14 +461,12 @@ public class Attribute extends Node {
      * </ul>
      */
     public void setNamespace(String prefix, String URI) {
-        
-        if ("xml".equals(prefix) && "id".equals(this.localName)) {
-            Verifier.checkNCName(this.value);
+               
+        _setNamespace(prefix, URI);
+        if (isXMLID()) {
             this.setType(Attribute.Type.ID);
         }
-        
-        _setNamespace(prefix, URI);
-        
+
     }
 
     
@@ -484,7 +475,6 @@ public class Attribute extends Node {
         if (URI == null) URI = "";
         if (prefix == null) prefix = "";
         
-        // ???? use == to compare?
         if (prefix.equals("xmlns")) {
             throw new IllegalNameException(
               "Attribute objects are not used to represent "
@@ -1077,7 +1067,7 @@ public class Attribute extends Node {
             StringBuffer result 
               = new StringBuffer("[Attribute.Type: ");
             result.append(getName()); 
-            result.append("]");
+            result.append(']');
             return result.toString();    
             
         }         
