@@ -1,4 +1,4 @@
-/* Copyright 2002-2006, 2009 Elliotte Rusty Harold
+/* Copyright 2002-2006, 2009, 2010, 2013 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -53,7 +53,7 @@ import org.apache.xerces.impl.Version;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.2
+ * @version 1.2.10
  * 
  */
 public class Builder {
@@ -184,7 +184,8 @@ public class Builder {
         "net.sf.saxon.aelfred.SAXDriver",
         "com.icl.saxon.aelfred.SAXDriver",
         "org.dom4j.io.aelfred2.SAXDriver",
-        "org.dom4j.io.aelfred.SAXDriver"
+        "org.dom4j.io.aelfred.SAXDriver",
+        "org.xmlpull.v1.sax2.Driver" // android
     };
 
     
@@ -261,6 +262,13 @@ public class Builder {
     private static void setupParser(XMLReader parser, boolean validate)
       throws SAXNotRecognizedException, SAXNotSupportedException {
         
+        // General configuration for all parsers
+        parser.setFeature(
+                "http://xml.org/sax/features/namespace-prefixes", true);
+        parser.setFeature(
+                "http://xml.org/sax/features/namespaces", true);
+        
+        // Parser specific configuration
         XMLReader baseParser = parser;
         while (baseParser instanceof XMLFilter) {
              XMLReader parent = ((XMLFilter) baseParser).getParent();
@@ -269,10 +277,6 @@ public class Builder {
         }
         
         String parserName = baseParser.getClass().getName();
-        parser.setFeature(
-          "http://xml.org/sax/features/namespace-prefixes", true);
-        parser.setFeature(
-          "http://xml.org/sax/features/namespaces", true);
         if (!validate) {
             if (parserName.equals(  // Crimson workaround
               "org.apache.crimson.parser.XMLReaderImpl")) {
@@ -554,8 +558,7 @@ public class Builder {
      * that takes a <code>java.io.File</code> object as an argument.
      * </p>
      * 
-     * @param systemID the URL (generally absolute) 
-     *     from which the document is read.
+     * @param systemID an absolute URL from which the document is read.
      *     The URL's scheme must be one supported by the Java VM. 
      * 
      * @return the parsed <code>Document</code>
@@ -609,7 +612,7 @@ public class Builder {
      * </p>
      * 
      * @param in the input stream from which the document is read
-     * @param baseURI the base URI for this document
+     * @param baseURI an absolute URI for this document; may be null
      * 
      * @return the parsed <code>Document</code>
      * 
@@ -622,9 +625,11 @@ public class Builder {
     public Document build(InputStream in, String baseURI) 
       throws ParsingException, ValidityException, IOException {
 
-        baseURI = canonicalizeURL(baseURI);
         InputSource source = new InputSource(in);
-        source.setSystemId(baseURI);
+        if (baseURI != null) {
+            baseURI = canonicalizeURL(baseURI);
+            source.setSystemId(baseURI);
+        }
         return build(source);
         
     }
@@ -1029,6 +1034,7 @@ public class Builder {
     public Document build(Reader in) 
       throws ParsingException, ValidityException, IOException {
 
+        if (in == null) throw new NullPointerException("Attempted to build from null reader");
         InputSource source = new InputSource(in);
         return build(source);
         
@@ -1043,7 +1049,7 @@ public class Builder {
      * 
      * @param in the reader from which the document 
      *     is read
-     * @param baseURI the base URI for this document
+     * @param baseURI the base URI for this document; may be null
      * 
      * @return the parsed <code>Document</code>
      * 
@@ -1056,10 +1062,12 @@ public class Builder {
      */
     public Document build(Reader in, String baseURI) 
       throws ParsingException, ValidityException, IOException {
-          
-        baseURI = canonicalizeURL(baseURI);
+         
         InputSource source = new InputSource(in);
-        source.setSystemId(baseURI);
+        if (baseURI != null) {
+            baseURI = canonicalizeURL(baseURI);
+            source.setSystemId(baseURI);
+        }
         return build(source);
         
     }
@@ -1070,9 +1078,8 @@ public class Builder {
      * Reads the document from the contents of a string.
      * </p>
      * 
-     * @param document the string that contains 
-     *      the XML document. 
-     * @param baseURI the base URI for this document
+     * @param document the string that contains the XML document
+     * @param baseURI the base URI for this document; may be null
      * 
      * @return  the parsed <code>Document</code>
      * 
@@ -1097,24 +1104,20 @@ public class Builder {
     // Also needed to work around a VM bug involving file URLs such as
     // file:///tmp/nosuchdirectory/../foo.xml
     // where "nosuchdirectory" does not exist.
-    // ???? should this move into the URIUtil class?
     private String canonicalizeURL(String uri) {
         
         try {
             URL u = new URL(uri);
-            String path = u.getFile();
+            String path = u.getPath();
             String scheme = u.getProtocol();
             String authority = u.getHost();
             String query = u.getQuery();
             int port = u.getPort();
             // fragment ID not needed
-            if (path == null || path.length() == 0 
+            if (path == null || path.length() == 0) {
               // We handle here the case where we have a URL such as 
               // http://www.cafeaulait.org with no trailing slash.
-              // Java's URL class assigns the path "/" to this case
-              // but does not change the URL. 
-              || ("/".equals(path) && !(uri.endsWith("/")))) {
-                path += '/';
+                path = "/";
             }
             // If this proves to be a hot spot we could probably take this path
             // only if the scheme is file; not in the more common case where
@@ -1140,8 +1143,7 @@ public class Builder {
      * Reads the document from a SAX <code>InputSource</code>.
      * </p>
      * 
-     * @param in the input source from 
-     *     which the document is read. 
+     * @param in the input source from which the document is read
      * 
      * @return the parsed <code>Document</code>
      * 
