@@ -1,4 +1,4 @@
-/* Copyright 2002-2006 Elliotte Rusty Harold
+/* Copyright 2002-2006, 2011, 2013 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -15,7 +15,7 @@
    Boston, MA 02111-1307  USA
    
    You can contact Elliotte Rusty Harold by sending e-mail to
-   elharo@metalab.unc.edu. Please include the word "XOM" in the
+   elharo@ibiblio.org. Please include the word "XOM" in the
    subject line. The XOM home page is located at http://www.xom.nu/
 */
 
@@ -26,8 +26,6 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.NoSuchElementException;
-// ???? Why do I still need these imports? Could 
-// I get rid of them? 
 import java.util.Set;
 import java.util.LinkedHashSet;
 
@@ -48,7 +46,7 @@ import java.util.LinkedHashSet;
  * </ul>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.2
+ * @version 1.2.10
  *
  */
 public class Element extends ParentNode {
@@ -1842,32 +1840,33 @@ public class Element extends ParentNode {
     }
 
 
+    private class AttributeIterator implements Iterator {
+      
+      private int next = 0;
+      
+      public boolean hasNext() {
+          return next < numAttributes;
+      }
+
+      public Object next() throws NoSuchElementException {
+          
+          if (hasNext()) {
+              Attribute a = attributes[next];
+              next++;
+              return a;
+          }
+          throw new NoSuchElementException("No such attribute");
+          
+      }
+
+      public void remove() {
+          throw new UnsupportedOperationException();
+      }
+    }
+    
     Iterator attributeIterator() {
 
-        return new Iterator() {
-
-            private int next = 0;
-            
-            public boolean hasNext() {
-                return next < numAttributes;
-            }
-
-            public Object next() throws NoSuchElementException {
-                
-                if (hasNext()) {
-                    Attribute a = attributes[next];
-                    next++;
-                    return a;
-                }
-                throw new NoSuchElementException("No such attribute");
-                
-            }
-
-            public void remove() {
-                throw new UnsupportedOperationException();
-            }
-            
-        };
+        return new AttributeIterator();
     }
 
     
