@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -26,9 +26,8 @@ import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
-import java.util.Iterator;
 import java.util.Locale;
-import java.util.Map;
+import org.xml.sax.helpers.NamespaceSupport;
 
 /**
  * <p>
@@ -47,13 +46,15 @@ import java.util.Map;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b7
+ * @version 1.2d1
  * 
  */
 public class Serializer {
 
     private TextWriter escaper;
     private boolean preserveBaseURI = false;
+    // ???? reset when exception is thrown?
+    private NamespaceSupport namespaces = new NamespaceSupport();
 
     
     /**
@@ -242,6 +243,8 @@ public class Serializer {
     public void write(Document doc) throws IOException {
         
         escaper.reset();
+        namespaces.reset();
+        namespaces.declarePrefix("", "");
         // The OutputStreamWriter automatically inserts
         // the byte order mark if necessary.
         writeXMLDeclaration();
@@ -270,9 +273,9 @@ public class Serializer {
      */
     protected void writeXMLDeclaration() throws IOException {
         
-        escaper.writeMarkup("<?xml version=\"1.0\" encoding=\"");
-        escaper.writeMarkup(escaper.getEncoding());
-        escaper.writeMarkup("\"?>");
+        escaper.writeUncheckedMarkup("<?xml version=\"1.0\" encoding=\"");
+        escaper.writeUncheckedMarkup(escaper.getEncoding());
+        escaper.writeUncheckedMarkup("\"?>");
         escaper.breakLine();
         
     }
@@ -281,9 +284,7 @@ public class Serializer {
     /**
      * <p>
      * Serializes an element onto the output stream using the current
-     * options. The result is guaranteed to be well-formed. If 
-     * <code>element</code> does not have a parent element, the output  
-     * will also be namespace well-formed.
+     * options. The result is guaranteed to be well-formed.
      * </p>
      * 
      * <p>
@@ -300,8 +301,8 @@ public class Serializer {
      * </ol>
      * 
      * <p>
-     *   It may break lines or add white space if the serializer has
-     *   been configured to indent or use a maximum line length.
+     * It may break lines or add white space if the serializer has
+     * been configured to indent or use a maximum line length.
      * </p>
      * 
      * @param element the <code>Element</code> to serialize
@@ -314,13 +315,6 @@ public class Serializer {
      */
     protected void write(Element element) throws IOException {
 
-        boolean wasPreservingWhiteSpace = escaper.isPreserveSpace();
-        if (escaper.isIndenting() 
-          && !wasPreservingWhiteSpace 
-          && !escaper.justBroke()) {
-            escaper.breakLine();
-        }
-        
         // workaround for case where only children are empty text nodes
         boolean hasRealChildren = false;
         int childCount = element.getChildCount();
@@ -335,20 +329,9 @@ public class Serializer {
         }
         
         if (hasRealChildren) {
+            boolean wasPreservingWhiteSpace = escaper.isPreserveSpace();
             writeStartTag(element);
-            // adjust for xml:space
-            String newXMLSpaceValue = element.getAttributeValue(
-              "space", "http://www.w3.org/XML/1998/namespace");
-            if  (newXMLSpaceValue != null) {
-                if ("preserve".equals(newXMLSpaceValue)){
-                    escaper.setPreserveSpace(true);
-                }
-                else if ("default".equals(newXMLSpaceValue)){
-                    escaper.setPreserveSpace(false);
-                }
-            }
             
-            escaper.incrementIndent();
             // children
             for (int i = 0; i < childCount; i++) {
                 Node child = element.getChild(i);
@@ -371,24 +354,15 @@ public class Serializer {
                     writeChild(child);
                 }
             }
-            escaper.decrementIndent();
-            if (escaper.getIndent() > 0 && !escaper.isPreserveSpace()) {
-                if (hasNonTextChildren(element)) {
-                    escaper.breakLine();
-                }
-            }
             writeEndTag(element);
             
             // restore parent value
-            if  (newXMLSpaceValue != null) {
-                escaper.setPreserveSpace(wasPreservingWhiteSpace);
-            }
-                        
+            escaper.setPreserveSpace(wasPreservingWhiteSpace);
         }
         else {
             writeEmptyElementTag(element);   
         }
-        
+                
     }
 
     
@@ -418,9 +392,19 @@ public class Serializer {
      *     encounters an I/O error
      */
     protected void writeEndTag(Element element) throws IOException {
-        escaper.writeMarkup("</");
-        escaper.writeMarkup(element.getQualifiedName());
-        escaper.writeMarkup('>');
+        
+        escaper.decrementIndent();
+        if (escaper.getIndent() > 0 && !escaper.isPreserveSpace()) {
+            if (hasNonTextChildren(element)) {
+                escaper.breakLine();
+            }
+        }
+        escaper.write('<');
+        escaper.write('/');
+        escaper.writeName(element.getQualifiedName());
+        escaper.write('>');
+        namespaces.popContext();
+        
     }
 
     
@@ -447,8 +431,21 @@ public class Serializer {
      *     that is not available in the current encoding
      */
     protected void writeStartTag(Element element) throws IOException {
+        
         writeTagBeginning(element);
-        escaper.writeMarkup('>');
+        escaper.write('>');
+        escaper.incrementIndent();
+        String xmlSpaceValue = element.getAttributeValue(
+           "space", "http://www.w3.org/XML/1998/namespace");
+        if  (xmlSpaceValue != null) {
+            if ("preserve".equals(xmlSpaceValue)){
+                escaper.setPreserveSpace(true);
+            }
+            else if ("default".equals(xmlSpaceValue)){
+                escaper.setPreserveSpace(false);
+            }
+        }
+        
     }
 
     
@@ -484,7 +481,9 @@ public class Serializer {
     protected void writeEmptyElementTag(Element element) 
       throws IOException {
         writeTagBeginning(element);
-        escaper.writeMarkup("/>");
+        escaper.write('/');
+        escaper.write('>');
+        namespaces.popContext();
     }
 
     
@@ -492,10 +491,19 @@ public class Serializer {
     // and writeEmptyElementTag
     private void writeTagBeginning(Element element) 
       throws IOException {
-        escaper.writeMarkup('<');
-        escaper.writeMarkup(element.getQualifiedName());
+        
+        namespaces.pushContext();
+        
+        if (escaper.isIndenting() 
+          && !escaper.isPreserveSpace() 
+          && !escaper.justBroke()) {
+            escaper.breakLine();
+        }
+        escaper.write('<');
+        escaper.writeName(element.getQualifiedName());
         writeAttributes(element);           
         writeNamespaceDeclarations(element);
+        
     }
 
 
@@ -533,7 +541,7 @@ public class Serializer {
                   || !element.getBaseURI()
                        .equals(parent.getBaseURI())) {
                        
-                    escaper.writeMarkup(' ');
+                    escaper.write(' ');
                     Attribute baseAttribute = new Attribute(
                       "xml:base", 
                       "http://www.w3.org/XML/1998/namespace", 
@@ -546,7 +554,7 @@ public class Serializer {
         int attributeCount = element.getAttributeCount();
         for (int i = 0; i < attributeCount; i++) {
             Attribute attribute = element.getAttribute(i);
-            escaper.writeMarkup(' ');
+            escaper.write(' ');
             write(attribute);
         }  
     }
@@ -572,30 +580,50 @@ public class Serializer {
     protected void writeNamespaceDeclarations(Element element)
       throws IOException {
         
-        ParentNode parent = element.getParent();
-        
-        Map prefixes = element.getNamespacePrefixesInScope();
-        Iterator iterator = prefixes.entrySet().iterator();
-        while (iterator.hasNext()) {
-            Map.Entry entry = (Map.Entry) iterator.next();
-            String additionalPrefix = (String) entry.getKey();
-            String uri = (String) entry.getValue();
-            if (parent != null && parent.isElement()) {
-               Element parentElement = (Element) parent;   
-               if (uri.equals(
-                 parentElement.getNamespaceURI(additionalPrefix))) {
-                   continue;
-               }
-            }
-            else if (uri.equals("")) {
-                continue; // no need to say xmlns=""   
-            }
-            
-            // XXX replace with a writeSpace method????
-            escaper.writeMarkup(' ');
-            writeNamespaceDeclaration(additionalPrefix, uri);
+        String prefix = element.getNamespacePrefix();
+        if (!("xml".equals(prefix))) {
+            writeNamespaceDeclarationIfNecessary(prefix, element.getNamespaceURI());
         }
         
+        // write attribute namespaces
+        int attCount = element.getAttributeCount();
+        for (int i = 0; i < attCount; i++) {
+            Attribute att = element.getAttribute(i);
+            String attPrefix = att.getNamespacePrefix();
+            if (attPrefix.length() != 0 && !("xml".equals(attPrefix))) {
+                writeNamespaceDeclarationIfNecessary(attPrefix, att.getNamespaceURI());    
+            }
+        }
+        
+        // write additional namespaces
+        Namespaces namespaces = element.namespaces;
+        if (namespaces == null) return;
+        int namespaceCount = namespaces.size();
+        for (int i = 0; i < namespaceCount; i++) {
+            String additionalPrefix = namespaces.getPrefix(i);
+            String uri = namespaces.getURI(additionalPrefix);
+            writeNamespaceDeclarationIfNecessary(additionalPrefix, uri);
+        }
+        
+    }
+    
+    
+    private void writeNamespaceDeclarationIfNecessary(String prefix, String uri) 
+      throws IOException {
+        
+        String currentValue = namespaces.getURI(prefix);
+        // NamespaceSupport returns null for no namespace, not the 
+        // empty string like XOM does
+        if (currentValue == null && "".equals(uri)) {
+            return;
+        }
+        else if (uri.equals(currentValue)) {
+            return;
+        }
+        
+        escaper.write(' ');
+        writeNamespaceDeclaration(prefix, uri);      
+        
     }
 
 
@@ -620,16 +648,18 @@ public class Serializer {
     protected void writeNamespaceDeclaration(String prefix, String uri)
       throws IOException {
         
+        namespaces.declarePrefix(prefix, uri);
         if ("".equals(prefix)) {
-            escaper.writeMarkup("xmlns"); 
+            escaper.writeUncheckedMarkup("xmlns"); 
         }
         else {
-            escaper.writeMarkup("xmlns:"); 
-            escaper.writeMarkup(prefix); 
+            escaper.writeUncheckedMarkup("xmlns:"); 
+            escaper.writeName(prefix); 
         } 
-        escaper.writeMarkup("=\""); 
+        escaper.write('='); 
+        escaper.write('"'); 
         escaper.writePCDATA(uri);   
-        escaper.writeMarkup('\"');
+        escaper.write('"');
         
     }
 
@@ -650,10 +680,11 @@ public class Serializer {
      * 
      */
     protected void write(Attribute attribute) throws IOException {
-        escaper.writeMarkup(attribute.getQualifiedName());
-        escaper.writeMarkup("=\"");
+        escaper.writeName(attribute.getQualifiedName());
+        escaper.write('=');
+        escaper.write('"');
         escaper.writeAttributeValue(attribute.getValue());
-        escaper.writeMarkup('\"');  
+        escaper.write('"');  
     }
     
     
@@ -675,9 +706,9 @@ public class Serializer {
      */
     protected void write(Comment comment) throws IOException {
         if (escaper.isIndenting()) escaper.breakLine();
-        escaper.writeMarkup("<!--");
+        escaper.writeUncheckedMarkup("<!--");
         escaper.writeMarkup(comment.getValue());
-        escaper.writeMarkup("-->");
+        escaper.writeUncheckedMarkup("-->");
     }
     
     
@@ -704,16 +735,16 @@ public class Serializer {
       throws IOException {
         
         if (escaper.isIndenting()) escaper.breakLine();
-        escaper.writeMarkup("<?");
-        escaper.writeMarkup(instruction.getTarget());
+        escaper.writeUncheckedMarkup("<?");
+        escaper.writeName(instruction.getTarget());
         String value = instruction.getValue();
         // for canonical XML, only output a space after the target
         // if there is a value
         if (!"".equals(value)) {
-            escaper.writeMarkup(' ');
+            escaper.write(' ');
             escaper.writeMarkup(value);
         }
-        escaper.writeMarkup("?>"); 
+        escaper.writeUncheckedMarkup("?>"); 
         
     }
     
@@ -756,9 +787,9 @@ public class Serializer {
                    }   
                 }
             }
-            escaper.writeMarkup("<![CDATA[");
+            escaper.writeUncheckedMarkup("<![CDATA[");
             escaper.writeMarkup(value);
-            escaper.writeMarkup("]]>");
+            escaper.writeUncheckedMarkup("]]>");
         }
         // is this boundary whitespace we can ignore?
         else if (isBoundaryWhitespace(text, value)) {
@@ -828,8 +859,8 @@ public class Serializer {
      */
     protected void write(DocType doctype) throws IOException {
         
-        escaper.writeMarkup("<!DOCTYPE ");
-        escaper.writeMarkup(doctype.getRootElementName());
+        escaper.writeUncheckedMarkup("<!DOCTYPE ");
+        escaper.writeName(doctype.getRootElementName());
         if (doctype.getPublicID() != null) {
           escaper.writeMarkup(" PUBLIC \"" + doctype.getPublicID() 
            + "\" \"" + doctype.getSystemID() + "\"");
@@ -841,15 +872,15 @@ public class Serializer {
         
         String internalDTDSubset = doctype.getInternalDTDSubset();
         if (!internalDTDSubset.equals("")) {
-            escaper.writeMarkup(" [");    
+            escaper.writeUncheckedMarkup(" [");    
             escaper.breakLine();
             escaper.setInDocType(true);
             escaper.writeMarkup(internalDTDSubset); 
             escaper.setInDocType(false);
-            escaper.writeMarkup("]"); 
+            escaper.write(']'); 
         }
 
-        escaper.writeMarkup(">");
+        escaper.write('>');
         
     }   
 
@@ -1203,6 +1234,7 @@ public class Serializer {
         return escaper.getEncoding();   
     }
     
+    
     /**
      * <p>
      *   If true, this property indicates serialization will
