@@ -1,4 +1,4 @@
-/* Copyright 2002-2006 Elliotte Rusty Harold
+/* Copyright 2002-2006, 2009 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -15,7 +15,7 @@
    Boston, MA 02111-1307  USA
    
    You can contact Elliotte Rusty Harold by sending e-mail to
-   elharo@metalab.unc.edu. Please include the word "XOM" in the
+   elharo@ibiblio.org. Please include the word "XOM" in the
    subject line. The XOM home page is located at http://www.xom.nu/
 */
 
@@ -32,7 +32,7 @@ import org.xml.sax.ext.LexicalHandler;
 
 /**
  * @author Elliotte Rusty Harold
- * @version 1.2b1
+ * @version 1.2.3
  *
  */
 class XOMHandler 
@@ -450,7 +450,7 @@ class XOMHandler
         
         inDTD = false;
         if (doctype != null) {
-            doctype.fastSetInternalDTDSubset(internalDTDSubset.toString());
+            doctype.setInternalDTDSubset(internalDTDSubset.toString());
         }
         
     }
@@ -524,6 +524,9 @@ class XOMHandler
     }    
     
     
+    // TODO(elharo) an untrusted parser could push in bad names and 
+    // values in declaration in the internal DTD subset.
+    // Possibly declarations should be created in the factory too.
     public void elementDecl(String name, String model) {
         
         if (inInternalSubset() && doctype != null) {
@@ -548,9 +551,12 @@ class XOMHandler
     // It is not intended for use anywhere in the document.
     protected boolean inInternalSubset() {
 
-        if (!usingCrimson && !inExternalSubset) return true;
+        if (!usingCrimson) {
+            return !inExternalSubset;
+        }
         String currentURI = locator.getSystemId();
         if (currentURI == this.documentBaseURI) return true;
+        if (currentURI == null) return false;
         if (currentURI.equals(this.documentBaseURI)) return true;
         return false;
         
