@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006, 2009, 2011 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -15,7 +15,7 @@
    Boston, MA 02111-1307  USA
    
    You can contact Elliotte Rusty Harold by sending e-mail to
-   elharo@metalab.unc.edu. Please include the word "XOM" in the
+   elharo@ibiblio.org. Please include the word "XOM" in the
    subject line. The XOM home page is located at http://www.xom.nu/
 */
 
@@ -26,9 +26,11 @@ import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.SortedMap;
 import java.util.StringTokenizer;
 import java.util.TreeMap;
@@ -56,17 +58,18 @@ import nu.xom.XPathContext;
  *   href="http://www.w3.org/TR/2001/REC-xml-c14n-20010315">Canonical
  *   XML Version 1.0</a> or <a target="_top"
  *   href="http://www.w3.org/TR/2002/REC-xml-exc-c14n-20020718/">Exclusive
- *   XML Canonicalization Version 1.0</a>. 
+ *   XML Canonicalization Version 1.0</a>. Canonical XML 1.1 support is in progress. 
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b4
+ * @version 1.2.7
  *
  */
 public class Canonicalizer {
 
     private boolean withComments;
     private boolean exclusive = false;
+    private boolean v11 = false;
     private CanonicalXMLSerializer serializer;
     private List inclusiveNamespacePrefixes = new ArrayList();
     
@@ -81,6 +84,10 @@ public class Canonicalizer {
       "http://www.w3.org/2001/10/xml-exc-c14n#";
     public final static String EXCLUSIVE_XML_CANONICALIZATION_WITH_COMMENTS = 
       "http://www.w3.org/2001/10/xml-exc-c14n#WithComments";
+    public final static String CANONICAL_XML_11 =  
+      "http://www.w3.org/2006/12/xml-c14n11";
+    public final static String CANONICAL_XML_11_WITH_COMMENTS =  
+      "http://www.w3.org/2006/12/xml-c14n11#WithComments"; 
     
     
     private static class AttributeComparator implements Comparator {
@@ -218,6 +225,16 @@ public class Canonicalizer {
             this.withComments = true;
             this.exclusive = true;            
         }
+        else if (algorithm.equals(CANONICAL_XML_11)) {
+            this.withComments = false;
+            this.exclusive = false;
+            this.v11 = true;
+        }
+        else if (algorithm.equals(CANONICAL_XML_11_WITH_COMMENTS)) {
+            this.withComments = true;
+            this.exclusive = false;
+            this.v11 = true;
+        }
         else {
             throw new CanonicalizationException(
               "Unsupported canonicalization algorithm: " + algorithm);
@@ -470,8 +487,8 @@ public class Canonicalizer {
             Attribute[] sorted = sortAttributes(element);        
             for (int i = 0; i < sorted.length; i++) {
                 if (nodes == null || nodes.contains(sorted[i]) 
-                   || (sorted[i].getNamespaceURI().equals(Namespace.XML_NAMESPACE) 
-                       && sorted[i].getParent() != element)) {
+                   || (sorted[i].getNamespaceURI().equals(Namespace.XML_NAMESPACE) && sorted[i].getParent() != element)
+                   ) {
                     write(sorted[i]);
                 }
             }       
@@ -606,7 +623,7 @@ public class Canonicalizer {
             Map nearest = new TreeMap();
             // add in any inherited xml: attributes
             if (!exclusive && nodes != null && nodes.contains(element) 
-              && ! nodes.contains(element.getParent())) {
+              && !nodes.contains(element.getParent())) {
                 // grab all xml: attributes
                 Nodes attributes = element.query("ancestor::*/@xml:*", xmlcontext);
                 if (attributes.size() != 0) {
@@ -620,14 +637,18 @@ public class Canonicalizer {
                         if (element.getAttribute(name, Namespace.XML_NAMESPACE) != null) {
                             // this element already has that attribute
                             continue;
+                        } else if (v11 && "id".equals(name)) { // Don't inherit xml:id in C14N 1.1
+                            continue;
                         }
                         if (! nearest.containsKey(name)) {
-                            Element parent = (Element) a.getParent();
-                            if (! nodes.contains(parent)) {
-                                nearest.put(name, a);
-                            }
-                            else {
-                                nearest.put(name, null);
+                            if (!v11 || "lang".equals(name) || "space".equals(name)) {
+                                Element parent = (Element) a.getParent();
+                                if (! nodes.contains(parent)) {
+                                    nearest.put(name, a);
+                                }
+                                else {
+                                    nearest.put(name, null);
+                                }
                             }
                         }
                     }
@@ -636,10 +657,29 @@ public class Canonicalizer {
                 // remove null values
                 Iterator iterator = nearest.values().iterator();
                 while (iterator.hasNext()) {
-                    if (iterator.next() == null) iterator.remove();
+                    Attribute a = (Attribute) iterator.next();
+                    if (a == null) iterator.remove();
                 }
                 
+                if (v11) { // fixup xml:base attributes
+                    List bases = getOmittedBases(element);
+                    Attribute baseAttribute = element.getAttribute("base", Namespace.XML_NAMESPACE);
+                    String baseValue = "";
+                    if (baseAttribute != null) {
+                        baseValue = baseAttribute.getValue();
+                        element.removeAttribute(baseAttribute);
+                    }
+                    if (!bases.isEmpty()) {
+                      for (int i = 0; i < bases.size(); i++) {
+                          baseValue = joinURIReferences((String) bases.get(i), baseValue);
+                      }
+                    }
+                    if (baseValue != null && baseValue.length() > 0) {
+                        nearest.put("base", new Attribute("xml:base", Namespace.XML_NAMESPACE, baseValue));
+                    }
+                }
             }
+
             
             int localCount = element.getAttributeCount();
             Attribute[] result 
@@ -658,8 +698,23 @@ public class Canonicalizer {
             return result;        
             
         }
-    
-        
+
+
+        private List getOmittedBases(Element element) {
+            ArrayList bases = new ArrayList();
+            // TODO(elharo): rework this to not need this next variable
+            ParentNode parent = element.getParent();
+            while (parent != null && parent instanceof Element && !nodes.contains(parent)) {
+                Element parentElement = (Element) parent;
+                String base = parentElement.getAttributeValue("base", Namespace.XML_NAMESPACE);
+                if (base != null) bases.add(base);
+                parent = parentElement.getParent();
+            }
+            
+            return bases;
+        }
+
+
         private String prepareAttributeValue(Attribute attribute) {
     
             String value = attribute.getValue();
@@ -883,13 +938,40 @@ public class Canonicalizer {
             Element pseudoRoot = null;
             if (doc == null) {
                 pseudoRoot = new Element("pseudo");
-                doc = new Document(pseudoRoot);
+                new Document(pseudoRoot);
                 ParentNode root = (ParentNode) node;
                 while (root.getParent() != null) root = root.getParent();
                 pseudoRoot.appendChild(root);
             }
             try {
-                write(node.query(".//. | .//@* | .//namespace::*"));
+                // XXX Consider if it's faster to do this without XPath
+                Nodes nodes = node.query(".//. | .//@* | .//namespace::*");
+                if (exclusive) { // only include namespace nodes in scope
+                    Set prefixes = new HashSet(nodes.size());
+                    for (int i = 0; i < nodes.size(); i++) {
+                        Node n = nodes.get(i);
+                        if (n instanceof Element) {
+                            String prefix = ((Element) n).getNamespacePrefix();
+                            prefixes.add(prefix);
+                        }
+                        else if (n instanceof Attribute) {
+                            String prefix = ((Attribute) n).getNamespacePrefix();
+                            if (! "".equals(prefix)) prefixes.add(prefix);
+                        }
+                    }
+                    // run through the namespaces and remove any that aren't visibly utilized
+                    for (int i = 0; i < nodes.size(); i++) {
+                        Node n = nodes.get(i);
+                        if (n instanceof Namespace) {
+                            String prefix = ((Namespace) n).getPrefix();
+                            if (! prefixes.contains((prefix))) {
+                                nodes.remove(i);
+                                i--;
+                            }
+                        }
+                    }
+                }
+                write(nodes);
             }
             finally {
                 if (pseudoRoot != null) pseudoRoot.removeChild(0);
@@ -1018,6 +1100,17 @@ public class Canonicalizer {
         }
         
     }
+    
+    private static String joinURIReferences(String parent, String child) {
+        if ("..".equals(parent)) parent = "../";
+        if ("..".equals(child)) child = "../";
+
+        if (child == null) {
+            return parent;
+        } 
+        //return parent + child;
+        return URIJoiner.absolutize(parent, child);
+    }
 
 
     private static void sort(List in, List namespaces, Nodes out, ParentNode parent) {
