@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -27,6 +27,7 @@ import nu.xom.DocType;
 import nu.xom.Document;
 import nu.xom.Element;
 import nu.xom.Node;
+import nu.xom.NodeFactory;
 import nu.xom.Nodes;
 import nu.xom.ParentNode;
 import nu.xom.ProcessingInstruction;
@@ -56,7 +57,7 @@ import org.w3c.dom.NodeList;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b1
+ * @version 1.2d1
  *
  */
 public class DOMConverter {
@@ -94,32 +95,85 @@ public class DOMConverter {
      *     XML document
      */
     public static Document convert(org.w3c.dom.Document domDocument) {
+        return convert(domDocument, new NodeFactory());
+    }
+
+
+    // ???? should this return a Nodes instead?
+    /**
+     * <p>
+     * Translates a DOM <code>org.w3c.dom.Document</code> object 
+     * into an equivalent <code>nu.xom.Document</code> object as 
+     * controlled by a factory.
+     * The original DOM document is not changed.
+     * Some DOM <code>Document</code> objects cannot 
+     * be serialized as namespace well-formed XML, and  
+     * thus cannot be converted to XOM.
+     * </p>
+     * 
+     * @param domDocument the DOM document to translate
+     * @param factory the factory that converts each DOM node into
+     *    zero or more XOM nodes
+     *
+     * @return a XOM document
+     * 
+     * @throws XMLException if the DOM document is not a well-formed 
+     *     XML document
+     */
+    public static Document convert(
+      org.w3c.dom.Document domDocument, NodeFactory factory) {
         
         org.w3c.dom.Element domRoot = domDocument.getDocumentElement();
-        Element xomRoot = convert(domRoot);
-        Document xomDocument = new Document(xomRoot);
+        Element xomRoot = factory.makeRootElement(
+          domRoot.getTagName(), domRoot.getNamespaceURI());
+        Document xomDocument = factory.startMakingDocument();
+        xomDocument.setRootElement(xomRoot);
+        Nodes result = convert(domRoot, factory);
+        boolean beforeRoot = true;
+        int p = 0;
+        for (int i = 0; i < result.size(); i++) {
+            Node n = result.get(i);
+            if (beforeRoot) {
+                if (n instanceof Element) {
+                    xomDocument.setRootElement((Element) n);
+                    beforeRoot = false;
+                }
+                else {
+                    xomDocument.insertChild(n, p++);
+                }
+            }
+            else {
+                xomDocument.appendChild(n);
+            }
+        }
         
         org.w3c.dom.Node current = domDocument.getFirstChild();
         
         // prolog
         for (int position = 0; 
              current.getNodeType() != org.w3c.dom.Node.ELEMENT_NODE; 
-             position++, current = current.getNextSibling()) {
-            xomDocument.insertChild(convert(current), position);
+             current = current.getNextSibling()) {
+            Nodes nodes = convert(current, factory);
+            // FIXME fix for multiples
+            for (int i = 0; i < nodes.size(); i++) {
+                xomDocument.insertChild(nodes.get(i), position++);
+            }
         }
         // root element       
         current = current.getNextSibling();
         
         // epilog
         while (current != null) {
-            xomDocument.appendChild(convert(current));
+            Nodes nodes = convert(current, factory);
+            for (int i = 0; i < nodes.size(); i++) {
+                xomDocument.appendChild(nodes.get(i));
+            }
             current = current.getNextSibling();   
         }       
                        
         return xomDocument;
-        
     }
-
+    
     
     /**
      * <p>
@@ -140,34 +194,68 @@ public class DOMConverter {
      *     XML fragment
      */
     public static Nodes convert(DocumentFragment fragment) {
+        return convert(fragment, new NodeFactory());
+    }
+
+    
+    /**
+     * <p>
+     * Translates a DOM <code>org.w3c.dom.DocumentFragment</code>  
+     * object into an equivalent <code>nu.xom.Nodes</code> object,
+     * converting each DOM node as specified by a factory.
+     * The original DOM document fragment is not changed.
+     * Some DOM <code>DocumentFragment</code> objects cannot 
+     * be serialized as namespace well-balanced XML, and  
+     * thus cannot be converted to XOM.
+     * </p>
+     * 
+     * @param fragment the DOM document fragment to translate
+     * @param factory the NodeFactory that converts each DOM node into
+     *     a XOM node
+     * 
+     * @return a <code>Nodes</code> containing the converted 
+     *     fragment members
+     * 
+     * @throws XMLException if the DOM object is not a well-balanced 
+     *     XML fragment
+     */
+    public static Nodes convert(DocumentFragment fragment, NodeFactory factory) {
         
         Nodes result = new Nodes();  
         NodeList children = fragment.getChildNodes();
         for (int i = 0; i < children.getLength(); i++) {
-            result.append(convert(children.item(i)));
+            Nodes nodes = convert(children.item(i), factory);
+            appendNodes(result, nodes);
         }
      
         return result;
         
     }
+    
+    
+    private static void appendNodes(Nodes parent, Nodes children) {
+        for (int i = 0; i < children.size(); i++) {
+            parent.append(children.get(i));
+        } 
+    }
 
     
-    private static Node convert(org.w3c.dom.Node node) {
+    private static Nodes convert(org.w3c.dom.Node node, NodeFactory factory) {
         
         int type = node.getNodeType();
         switch (type) {
             case org.w3c.dom.Node.ELEMENT_NODE:
-                return convert((org.w3c.dom.Element) node);   
+                return convert((org.w3c.dom.Element) node, factory);   
             case org.w3c.dom.Node.COMMENT_NODE:
-                return convert((org.w3c.dom.Comment) node);   
+                return convert((org.w3c.dom.Comment) node, factory);   
             case org.w3c.dom.Node.DOCUMENT_TYPE_NODE:
-                return convert((org.w3c.dom.DocumentType) node);   
+                return convert((org.w3c.dom.DocumentType) node, factory);   
             case org.w3c.dom.Node.TEXT_NODE:
-                return convert((org.w3c.dom.Text) node);
+                return convert((org.w3c.dom.Text) node, factory);
             case org.w3c.dom.Node.CDATA_SECTION_NODE:
-                return convert((org.w3c.dom.Text) node);
+                return convert((org.w3c.dom.Text) node, factory);
             case org.w3c.dom.Node.PROCESSING_INSTRUCTION_NODE:
-                return convert((org.w3c.dom.ProcessingInstruction) node);
+                return convert((org.w3c.dom.ProcessingInstruction) node, factory);
             default:   
                 throw new XMLException(
                   "Unexpected DOM node type: " + type);
@@ -196,6 +284,10 @@ public class DOMConverter {
         return new Comment(comment.getNodeValue());
     }
 
+    private static Nodes convert(org.w3c.dom.Comment comment, NodeFactory factory) {       
+        return factory.makeComment(comment.getNodeValue());
+    }
+
     /**
      * <p>
      * Translates a DOM <code>org.w3c.dom.Text</code> object 
@@ -218,6 +310,11 @@ public class DOMConverter {
     }
 
     
+    private static Nodes convert(org.w3c.dom.Text text, NodeFactory factory) {       
+        return factory.makeText(text.getNodeValue());
+    }
+
+    
     /**
      * <p>
      * Translates a DOM <code>org.w3c.dom.Attr</code> object 
@@ -249,7 +346,6 @@ public class DOMConverter {
         
     }
 
-
     /**
      * <p>
      * Translates a DOM <code>org.w3c.dom.ProcessingInstruction</code> 
@@ -270,8 +366,13 @@ public class DOMConverter {
      */
     public static ProcessingInstruction convert(
         org.w3c.dom.ProcessingInstruction pi) {
-        return new ProcessingInstruction(
-          pi.getTarget(), pi.getNodeValue());
+        return new ProcessingInstruction(pi.getTarget(), pi.getNodeValue());
+    }
+
+    
+    private static Nodes convert(
+        org.w3c.dom.ProcessingInstruction pi, NodeFactory factory) {
+        return factory.makeProcessingInstruction(pi.getTarget(), pi.getNodeValue());
     }
 
     
@@ -304,6 +405,25 @@ public class DOMConverter {
     }
 
     
+    private static Nodes convert(org.w3c.dom.DocumentType doctype, NodeFactory factory) {
+
+        Nodes nodes = factory.makeDocType(
+                doctype.getName(),
+                doctype.getPublicId(),
+                doctype.getSystemId());
+        for (int i = 0; i < nodes.size(); i++) {
+            Node node = nodes.get(i);
+            if (node instanceof DocType) {
+                ((DocType) node).setInternalDTDSubset(doctype.getInternalSubset());
+                break;
+            }
+        }
+        
+        return nodes;
+
+    }
+
+    
     /**
      * <p>
      * Translates a DOM <code>org.w3c.dom.Element</code> 
@@ -320,9 +440,28 @@ public class DOMConverter {
      *     is not a well-formed XML element
      */
     public static Element convert(org.w3c.dom.Element element) {
+        return (Element) convert(element, new NodeFactory()).get(0);
+    }
+ 
+    
+    private static Nodes convert(org.w3c.dom.Element element, NodeFactory factory) {
         
         org.w3c.dom.Node current = element;
-        Element result = makeElement(element);
+        Element result = makeElement(element, factory);
+        
+        if (result == null) {
+            Nodes out = new Nodes();   
+            NodeList children = element.getChildNodes();
+            for (int i = 0; i < children.getLength(); i++) {
+                Nodes temp = convert(children.item(i), factory);
+                for (int j = 0; j < temp.size(); j++) {
+                    out.append(temp.get(j));
+                }
+            }
+            return out;
+        }
+        
+        
         ParentNode parent = result;
         boolean backtracking = false;
         while (true) {
@@ -340,33 +479,46 @@ public class DOMConverter {
             else {
                 current = current.getParentNode();
                 backtracking = true;
-                parent = parent.getParent();
+                if (parent.getParent() != null) parent = parent.getParent();
                 continue;
             }
             
             int type = current.getNodeType();
             if (type == org.w3c.dom.Node.ELEMENT_NODE) {
-                Element child = makeElement((org.w3c.dom.Element) current);
-                parent.appendChild(child);
-                if (current.hasChildNodes()) parent = child;
+                Element child = makeElement((org.w3c.dom.Element) current, factory);
+                if (child != null) {
+                    parent.appendChild(child);
+                    if (current.hasChildNodes()) parent = child;
+                }
             }
             else {
-                Node child = convert(current);
-                parent.appendChild(child);
+                Nodes children = convert(current, factory);
+                for (int i = 0; i < children.size(); i++) {
+                    parent.appendChild(children.get(i));
+                }
             }
             
         }
         
-        return result;  
+        return factory.finishMakingElement(result);
         
-    }
- 
+    }    
     
-    private static Element makeElement(org.w3c.dom.Element element) {
+    
+    private static Element makeElement(org.w3c.dom.Element element, NodeFactory factory) {
         
         String namespaceURI = element.getNamespaceURI();
         String tagName = element.getTagName();
-        Element result = new Element(tagName, namespaceURI);
+        
+        Element result;
+        if (element.getParentNode() == null 
+          || element.getParentNode().getNodeType() == org.w3c.dom.Node.DOCUMENT_NODE) {
+            result = factory.makeRootElement(tagName, namespaceURI);
+        }
+        else {
+            result = factory.startMakingElement(tagName, namespaceURI);
+        }
+        if (result == null) return null;
         
         // fill element's attributes and additional namespace declarations
         NamedNodeMap attributes = element.getAttributes();
@@ -384,8 +536,13 @@ public class DOMConverter {
                     result.addNamespaceDeclaration(prefix, value); 
                 }
             }
-            else { 
-                result.addAttribute(new Attribute(name, uri, value));
+            else {
+                Nodes nodes = factory.makeAttribute(name, uri, value, Attribute.Type.UNDECLARED);
+                for (int j = 0; j < nodes.size(); j++) {
+                    Node child = nodes.get(j);
+                    if (child instanceof Attribute) result.addAttribute((Attribute) child);
+                    else result.appendChild(child);
+                }
             }
         }
         return result;
