@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -37,6 +37,10 @@ import nu.xom.Comment;
 import nu.xom.DocType;
 import nu.xom.Document;
 import nu.xom.Element;
+import nu.xom.Elements;
+import nu.xom.Namespace;
+import nu.xom.Node;
+import nu.xom.NodeFactory;
 import nu.xom.Nodes;
 import nu.xom.ParsingException;
 import nu.xom.ProcessingInstruction;
@@ -44,6 +48,7 @@ import nu.xom.Text;
 import nu.xom.XMLException;
 import nu.xom.converters.DOMConverter;
 
+import org.w3c.dom.Attr;
 import org.w3c.dom.CDATASection;
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.DocumentFragment;
@@ -58,7 +63,7 @@ import org.xml.sax.SAXException;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b2
+ * @version 1.2b2
  *
  */
 public class DOMConverterTest extends XOMTestCase {
@@ -180,7 +185,7 @@ public class DOMConverterTest extends XOMTestCase {
         assertEquals("test", root.getLocalName());
         assertEquals("", root.getNamespaceURI());
         
-        assertTrue(doctype != null);
+        assertNotNull(doctype);
         assertTrue(xomDoc.getChild(0) instanceof DocType);
         assertTrue(xomDoc.getChild(4) instanceof nu.xom.Comment);
         assertTrue(xomDoc.getChild(2) instanceof nu.xom.Comment);
@@ -622,10 +627,23 @@ public class DOMConverterTest extends XOMTestCase {
         org.w3c.dom.Document domDoc = DOMConverter.convert(doc, impl);
         org.w3c.dom.Element domRoot = domDoc.getDocumentElement();
         assertEquals(1, domRoot.getAttributes().getLength());
-         
+        assertEquals("preserve", domRoot.getAttributeNS("http://www.w3.org/XML/1998/namespace", "space"));
+        
     }
  
     
+    public void testConvertXMLSpaceAttributeFromDOMToXOM() 
+      throws SAXException, IOException, ParserConfigurationException {
+   
+        DocumentBuilderFactory df = DocumentBuilderFactory.newInstance();
+        df.setNamespaceAware(true);
+        org.w3c.dom.Document doc = df.newDocumentBuilder().parse(new InputSource(new StringReader("<root xml:space='preserve'/>")));
+        Document xom = DOMConverter.convert (doc);
+        assertEquals("preserve", xom.getRootElement().getAttributeValue("space", Namespace.XML_NAMESPACE));
+        
+    }
+  
+
     public void testConvertXMLPrefixedElementFromXOMToDOM() 
       throws SAXException, IOException {
      
@@ -654,5 +672,67 @@ public class DOMConverterTest extends XOMTestCase {
         
         assertEquals(xomDocIn, xomDocOut);
         
-    }    
+    }
+    
+    public void testUseFactory() {
+
+        Document xomDocOut = DOMConverter.convert(domDocument, new ANodeFactory());
+        assertTrue(xomDocOut instanceof ADocument);
+        Element root = xomDocOut.getRootElement();
+        assertTrue(root instanceof AElement);
+        Node doctype = xomDocOut.getChild(0);
+        assertTrue(doctype instanceof ADocType);
+        Node pi = xomDocOut.getChild(1);
+        assertTrue(pi instanceof AProcessingInstruction);
+        Node comment = xomDocOut.getChild(2);
+        assertTrue(comment instanceof AComment);
+        assertTrue(root.getChild(0) instanceof AText);
+        Elements children = root.getChildElements();
+        for (int i = 0; i < children.size(); i++) {
+            Element child = children.get(i);
+            assertTrue(child instanceof AElement);
+            for (int j = 0; j < child.getAttributeCount(); j++) {
+                assertTrue(child.getAttribute(j) instanceof AnAttribute);
+            }
+        }
+        
+    }
+    
+    public void testUseFactoryWithPrologAndEpilog() {
+
+        org.w3c.dom.Comment before = domDocument.createComment("before");
+        org.w3c.dom.Comment after = domDocument.createComment("after");
+        domDocument.appendChild(after);
+        domDocument.insertBefore(before, domDocument.getFirstChild());
+        Document xomDocOut = DOMConverter.convert(domDocument, new ANodeFactory());
+        assertTrue(xomDocOut instanceof ADocument);
+        Comment bc = (Comment) xomDocOut.getChild(0);
+        Comment ac = (Comment) xomDocOut.getChild(xomDocOut.getChildCount()-1);
+        assertTrue(bc instanceof AComment);
+        assertTrue(ac instanceof AComment);
+        
+    }
+    
+    public void testUseMinimizingFactory() {
+        Document xomDocOut = DOMConverter.convert(
+          domDocument, new NodeFactoryTest.MinimizingFactory());
+        assertEquals(0, xomDocOut.getRootElement().getChildCount());    
+    }
+    
+    public void testUseStrippingFactory() {
+        Document xomDocOut = DOMConverter.convert(
+          domDocument, new StrippingFactory());
+        assertEquals("Hello dear\nvery importanthere's the link\n"
+     + "text in a namespace\ntext in a namespace", xomDocOut.getRootElement().getValue());    
+    }
+    
+    private static class StrippingFactory extends NodeFactory {
+        
+        public Element startMakingElement(String name, String namespaceURI) {
+            if (! "test".equals(name)) return null;
+            return super.startMakingElement(name, namespaceURI);
+        }
+        
+    }
+    
 }
