@@ -1,4 +1,4 @@
-/* Copyright 2002-2004 Elliotte Rusty Harold
+/* Copyright 2002-2004, 2011 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -15,7 +15,7 @@
    Boston, MA 02111-1307  USA
    
    You can contact Elliotte Rusty Harold by sending e-mail to
-   elharo@metalab.unc.edu. Please include the word "XOM" in the
+   elharo@mibiblio.org. Please include the word "XOM" in the
    subject line. The XOM home page is located at http://www.xom.nu/
 */
 
@@ -50,7 +50,7 @@ import nu.xom.Text;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.2
+ * @version 1.2.7
  *
  */
 public class ElementTest extends XOMTestCase {
@@ -285,7 +285,7 @@ public class ElementTest extends XOMTestCase {
         Element root = doc.getRootElement();
         String  form = root.toXML();
         Document doc2 
-          = builder.build(form, f.toURL().toExternalForm());
+          = builder.build(form, f.toURI().toString());
         Element root2 = doc2.getRootElement();
         assertEquals(root, root2);
          
@@ -316,6 +316,64 @@ public class ElementTest extends XOMTestCase {
         assertEquals("", e.getNamespaceURI());
         
     }
+    
+    
+   public void testInsertChildAfterEnd() {
+        
+        Element parent = new Element("p");
+        for (int i = 0; i < 3; i++) {
+            Element c = new Element("c");
+            parent.appendChild(c);
+        }
+        assertEquals(3, parent.getChildCount());
+        Element d = new Element("d");
+        
+        try {
+          parent.insertChild(d, 5);
+          fail("Inserted child too far");
+        } 
+        catch (IndexOutOfBoundsException ex) {
+            assertNotNull(ex.getMessage());
+        }
+        
+    }
+
+
+   public void testInsertChildAtEnd() {
+       
+       Element parent = new Element("p");
+       for (int i = 0; i < 3; i++) {
+           Element c = new Element("c");
+           parent.appendChild(c);
+       }
+       assertEquals(3, parent.getChildCount());
+       Element d = new Element("d");
+       
+       parent.insertChild(d, 3);
+       assertEquals(4, parent.getChildCount());
+       
+   }
+
+
+   public void testInsertChildNegative() {
+       
+       Element parent = new Element("p");
+       for (int i = 0; i < 3; i++) {
+           Element c = new Element("c");
+           parent.appendChild(c);
+       }
+       assertEquals(3, parent.getChildCount());
+       Element d = new Element("d");
+       
+       try {
+         parent.insertChild(d, -1);
+         fail("Inserted child with negative index");
+       } 
+       catch (IndexOutOfBoundsException ex) {
+           assertNotNull(ex);
+       }
+       
+   }
 
 
     public void testCantInsertDoctype() {
