@@ -1,4 +1,4 @@
-/* Copyright 2002-2005 Elliotte Rusty Harold
+/* Copyright 2002-2006, 2009 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -54,7 +54,7 @@ import java.io.UnsupportedEncodingException;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.1b7
+ * @version 1.2
  *
  */
 public class SerializerTest extends XOMTestCase {
@@ -119,6 +119,36 @@ public class SerializerTest extends XOMTestCase {
             
     }
     
+    
+    public void testNamespaceURIContainsAmpersand() 
+      throws ParsingException, IOException {
+
+        String input = "<a xmlns=\"http://www.example.com?foo=bar&amp;g=h\"/>\r\n";
+        String expected = "<?xml version=\"1.0\" encoding=\"US-ASCII\"?>\r\n" + input;
+        Document doc = parser.build(input, null);
+        Serializer serializer = new Serializer(out, "US-ASCII");
+        serializer.write(doc);
+        serializer.flush();
+        String result = out.toString("US-ASCII");
+        assertEquals(expected, result);
+        
+    }
+
+
+    public void testNamespaceURIContainsAmpersandPhilip() 
+      throws ParsingException, IOException {
+
+        Element root = new Element("x", "x:&");
+        Document doc = new Document(root);
+        String expected = "<?xml version=\"1.0\" encoding=\"US-ASCII\"?>\r\n<x xmlns=\"x:&amp;\"/>\r\n";
+        Serializer serializer = new Serializer(out, "US-ASCII");
+        serializer.write(doc);
+        serializer.flush();
+        String result = out.toString("US-ASCII");
+        assertEquals(expected, result);
+        
+    }
+
 
     public void testMultipleCombiningCharactersWithDifferentCombiningClassesNFC() 
       throws ParsingException, IOException {
@@ -214,6 +244,40 @@ public class SerializerTest extends XOMTestCase {
     }
 
     
+    public void testElementBeginsWithXML() 
+      throws ParsingException, IOException {
+    
+        String input = "<xml:tree/>";
+        String output = "<?xml version=\"1.0\" encoding=\"US-ASCII\"?>\r\n<xml:tree/>\r\n";  
+        
+        Document doc = parser.build(input, null);
+        Serializer serializer = new Serializer(out, "US-ASCII");
+        serializer.setUnicodeNormalizationFormC(true);
+        serializer.write(doc);
+        serializer.flush();
+        String result = out.toString("US-ASCII");
+        assertEquals(output, result);
+            
+    }
+
+    
+    public void testElementHasAdditionalNamespaces() 
+      throws ParsingException, IOException {
+    
+        String input = "<tree xmlns:foo='http://www.example.org/'/>";
+        String output = "<?xml version=\"1.0\" encoding=\"US-ASCII\"?>\r\n<tree xmlns:foo=\"http://www.example.org/\"/>\r\n";  
+        
+        Document doc = parser.build(input, null);
+        Serializer serializer = new Serializer(out, "US-ASCII");
+        serializer.setUnicodeNormalizationFormC(true);
+        serializer.write(doc);
+        serializer.flush();
+        String result = out.toString("US-ASCII");
+        assertEquals(output, result);
+            
+    }
+
+    
     public void testOjeon() 
       throws ParsingException, IOException {
     
@@ -720,6 +784,38 @@ public class SerializerTest extends XOMTestCase {
     }
 
     
+    public void testChildElementInNoNamespaceParentInDefaultNamespace() 
+      throws IOException, ParsingException { 
+
+        Element root = new Element("root", "http://www.example.org/");
+        Element child1 = new Element("child");
+        root.appendChild(child1);
+        Document doc = new Document(root);
+        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        Serializer serializer = new Serializer(out);
+        serializer.write(doc);
+        serializer.flush();
+        String s = out.toString("UTF-8");
+        assertTrue(s.indexOf("xmlns=\"\"") > 0);
+        
+    }
+    
+    
+    public void testSimpleSerialize() 
+      throws IOException, ParsingException { 
+
+        Element root = new Element("root");
+        Document doc = new Document(root);
+        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        Serializer serializer = new Serializer(out);
+        serializer.write(doc);
+        serializer.flush();
+        String s = out.toString("UTF-8");
+        assertEquals("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n<root/>\r\n", s);
+        
+    }
+    
+    
     public void testPrologAndEpilog() 
       throws IOException, ParsingException { 
         
@@ -1526,6 +1622,73 @@ public class SerializerTest extends XOMTestCase {
         );
         
     }
+    
+    
+    public void testIndentingFromSubclassThatAvoidsWriteElement() 
+      throws IOException {
+        
+        Element items = new Element("itemSet");
+        items.appendChild(new Element("item1"));
+        items.appendChild(new Element("item2"));
+        Document doc = new Document(items);
+        Serializer serializer = new AvoidWriteElement(out);
+        serializer.setIndent(4);
+        serializer.write(doc);
+        serializer.flush();
+        out.close();
+        String result = new String(out.toByteArray(), "UTF-8");
+        assertEquals(
+          "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
+          + "<itemSet>\r\n    <item1></item1>\r\n    <item2></item2>\r\n"
+          + "</itemSet>\r\n", 
+          result
+        );
+        
+    }
+    
+    
+    public void testIndentingFromSubclassThatAvoidsWriteElementWithXMLSpace() 
+      throws IOException {
+        
+        Element items = new Element("itemSet");
+        items.addAttribute(new Attribute("xml:space", Namespace.XML_NAMESPACE, "preserve"));
+        items.appendChild(new Element("item1"));
+        items.appendChild(new Element("item2"));
+        Document doc = new Document(items);
+        Serializer serializer = new AvoidWriteElement(out);
+        serializer.setIndent(4);
+        serializer.write(doc);
+        serializer.flush();
+        out.close();
+        String result = new String(out.toByteArray(), "UTF-8");
+        assertEquals(
+          "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
+          + "<itemSet xml:space=\"preserve\"><item1></item1><item2></item2>"
+          + "</itemSet>\r\n", 
+          result
+        );
+        
+    }
+    
+    
+    private static class AvoidWriteElement extends Serializer {
+
+        public AvoidWriteElement(OutputStream out) {
+            super(out);
+        }
+        
+        protected void write(Element element) throws IOException {
+
+            writeStartTag(element);
+            for (int i = 0; i < element.getChildCount(); i++) {
+                Node child = element.getChild(i);
+                writeChild(child);
+            }
+            writeEndTag(element);
+            
+        }
+        
+    }     
 
     
     public void testIndentAndBreakBeforeComment() throws IOException {
@@ -1822,53 +1985,7 @@ public class SerializerTest extends XOMTestCase {
     }
 
 
-    public void testUnicodeNFCTestSuite() 
-      throws ParsingException, IOException {
-        
-        Builder builder = new Builder();
-        Document doc = builder.build("data/nfctests.xml");
-        Elements tests = doc.getRootElement().getChildElements("test");
-        int size = tests.size();
-        for (int i = 0; i < size; i++) {
-            // System.out.println(i);
-            Element test = tests.get(i);
-            test.detach();
-            Document testdoc = new Document(test);
-            ByteArrayOutputStream out = new ByteArrayOutputStream();
-            Serializer serializer = new Serializer(out);
-            serializer.setUnicodeNormalizationFormC(true);
-            serializer.write(testdoc);
-            serializer.flush();
-            String result = new String(out.toByteArray(), "UTF-8");
-            Document resultDoc = builder.build(result, null);
-            Element root = resultDoc.getRootElement();
-            String c1 = root.getAttributeValue("c1");
-            String c2 = root.getAttributeValue("c2");
-            String c3 = root.getAttributeValue("c3");
-            String c4 = root.getAttributeValue("c4");
-            String c5 = root.getAttributeValue("c5");
-            /* String v1 = root.getAttributeValue("v1");
-            String v2 = root.getAttributeValue("v2");
-            String v3 = root.getAttributeValue("v3");
-            String v4 = root.getAttributeValue("v4");
-            String v5 = root.getAttributeValue("v5"); */
-            
-            assertEquals(root.getValue(), c1, c2);
-            assertEquals(c2, c3);
-            // I'm not sure the v's are correct past the BMP
-            //assertEquals(root.getValue(), c1, v1);
-            // assertEquals(c1, v2);
-            // assertEquals(c1, v3);
-            
-            assertEquals(c4, c5);
-            // assertEquals(c4, v4);
-            // assertEquals(c4, v5);
-            
-        }
-        
-    }
 
-    
     public void testNFCInAttribute() throws IOException {
         
         root.addAttribute(new Attribute("c\u0327", "c\u0327")); // c with combining cedilla
@@ -2440,6 +2557,20 @@ public class SerializerTest extends XOMTestCase {
     }    
 
     
+    public void testWriteParentedElementInANamespace() throws IOException {
+
+        ElementSerializer serializer = new ElementSerializer(out, "UTF-8");
+        Element a = new Element("a", "http://www.example.org");
+        Element b = new Element("b", "http://www.example.org");
+        b.appendChild(a);
+        serializer.write(a);
+        serializer.flush();
+        String result = out.toString("UTF-8");
+        assertEquals("<a xmlns=\"http://www.example.org\"/>", result);
+        
+    }    
+
+    
     public void testWriteEscaped() throws IOException {
 
         ExposingSerializer serializer = new ExposingSerializer(out, "UTF-8");
@@ -2619,4 +2750,50 @@ public class SerializerTest extends XOMTestCase {
     }
 
     
+    public void testUnicodeNFCTestSuite() 
+      throws ParsingException, IOException {
+        
+        Builder builder = new Builder();
+        Document doc = builder.build("data/nfctests.xml");
+        Elements tests = doc.getRootElement().getChildElements("test");
+        int size = tests.size();
+        for (int i = 0; i < size; i++) {
+            // System.out.println(i);
+            Element test = tests.get(i);
+            test.detach();
+            Document testdoc = new Document(test);
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            Serializer serializer = new Serializer(out);
+            serializer.setUnicodeNormalizationFormC(true);
+            serializer.write(testdoc);
+            serializer.flush();
+            String result = new String(out.toByteArray(), "UTF-8");
+            Document resultDoc = builder.build(result, null);
+            Element root = resultDoc.getRootElement();
+            String c1 = root.getAttributeValue("c1");
+            String c2 = root.getAttributeValue("c2");
+            String c3 = root.getAttributeValue("c3");
+            String c4 = root.getAttributeValue("c4");
+            String c5 = root.getAttributeValue("c5");
+            /* String v1 = root.getAttributeValue("v1");
+            String v2 = root.getAttributeValue("v2");
+            String v3 = root.getAttributeValue("v3");
+            String v4 = root.getAttributeValue("v4");
+            String v5 = root.getAttributeValue("v5"); */
+            
+            assertEquals(root.getValue(), c1, c2);
+            assertEquals(c2, c3);
+            // I'm not sure the v's are correct past the BMP
+            //assertEquals(root.getValue(), c1, v1);
+            // assertEquals(c1, v2);
+            // assertEquals(c1, v3);
+            
+            assertEquals(c4, c5);
+            // assertEquals(c4, v4);
+            // assertEquals(c4, v5);
+            
+        }
+        
+    }
+
 }
