@@ -288,7 +288,8 @@ public class XOMTestCase extends TestCase {
      * qualified names, namespace URI, attributes,
      * declared namespaces, and children
      * are equal. Consecutive text node children are coalesced
-     * before the comparison is made. If the two nodes are not equal, 
+     * before the comparison is made. Empty text nodes are removed.
+     * If the two nodes are not equal, 
      * a <code>ComparisonFailure</code> is thrown with the given 
      * message.
      * </p>
@@ -375,6 +376,7 @@ public class XOMTestCase extends TestCase {
             Node child = element.getChild(i);
             if (child instanceof Text) {
                 if (previousWasText) return true;
+                else if ("".equals(child.getValue())) return true;
                 else previousWasText = true;
             }
             else {
@@ -412,32 +414,25 @@ public class XOMTestCase extends TestCase {
         }
         
         // now compare everything that isn't text using the original
-        // element objects
-        for (int i = 0; i < nonTextNodes; i++) {
-            Node expectedChild = getNonTextNode(expected, i);
-            Node actualChild = getNonTextNode(actual, i);
+        // element objects; we already know they have the same number
+        // of non-text nodes
+        int a = 0; // actualIndex
+        int e = expected.getChildCount();
+        for (int i = 0; i < e; i++) {
+            Node expectedChild = expected.getChild(i);
+            if (expectedChild instanceof Text) continue;            
+            // find the next nontext child of actual
+            Node actualChild = actual.getChild(a);
+            a++;
+            while (actualChild instanceof Text) {
+                actualChild = actual.getChild(a);
+                a++;
+            }
             assertEquals(message, expectedChild, actualChild);
         }
         
     }
 
-    
-    private static Node getNonTextNode(Element element, int index) {
-
-        int nonTextCount = 0;
-        int count = element.getChildCount();
-        for (int i = 0; i < count; i++) {
-            Node child = element.getChild(i);
-            if (! (child instanceof Text) ) {
-                if (nonTextCount == index) return child;
-                nonTextCount++;
-            }
-        }
-        throw new RuntimeException(
-          "Bug in XOMTestCase: this statement should not be reachable");
-        
-    }
-
     /* We only need to make an element that has the combined text
      * nodes, and something as a child placeholder.
      * It does need to have the other pieces. 
@@ -451,6 +446,7 @@ public class XOMTestCase extends TestCase {
         for (int i = 0; i < count; i++) {
             Node child = element.getChild(i);
             if (child instanceof Text) {
+                if ("".equals(child.getValue())) continue;
                 sb.setLength(0);
                 do {
                     sb.append(child.getValue());
