@@ -1,4 +1,4 @@
-/* Copyright 2005, 2006 Elliotte Rusty Harold
+/* Copyright 2005, 2006, 2012 Elliotte Rusty Harold
    
    This library is free software; you can redistribute it and/or modify
    it under the terms of version 2.1 of the GNU Lesser General Public 
@@ -15,7 +15,7 @@
    Boston, MA 02111-1307  USA
    
    You can contact Elliotte Rusty Harold by sending e-mail to
-   elharo@metalab.unc.edu. Please include the word "XOM" in the
+   elharo@ibiblio.org. Please include the word "XOM" in the
    subject line. The XOM home page is located at http://www.xom.nu/
 */
 
@@ -52,7 +52,7 @@ import nu.xom.XPathTypeException;
  * </p>
  * 
  * @author Elliotte Rusty Harold
- * @version 1.2b3
+ * @version 1.2.8
  *
  */
 public class XPathTest extends XOMTestCase {
@@ -2214,6 +2214,18 @@ public class XPathTest extends XOMTestCase {
         }
         
     }
+
+    
+    public void testFunctionWithDefaultNamespace() throws ParsingException, IOException {
+        
+        Element root = new Element("root", "http://www.example.org");
+        Document doc = new Document(root);
+        XPathContext context = XPathContext.makeNamespaceContext(doc.getRootElement());
+        context.addNamespace("pre", doc.getRootElement().getNamespaceURI());
+        Nodes query = doc.query("//*[not(self::pre:test)]", context);
+        assertEquals(1, query.size());
+        
+    }
     
 
     public void testWhiteSpaceIsAllowedBetweenTokens() {
